/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.ars.ars.mgmt;

import de.bsvrz.ars.ars.mgmt.simulation.SimulationResultData;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.Pid;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectCollection;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.dataIdentificationSettings.DataIdentification;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.util.K;

import java.util.*;

/**
 * Diese Klasse wertet die Parametrierung fuer Simulationen aus. Der Auswerte-Algorithmus ist von der Klasse {@link SettingsManager} der Kernsoftware
 * uebernommen.
 *
 * @author beck et al. projects GmbH
 * @author Alexander Schmidt
 * @version $Revision: 6419 $ / $Date: 2009-03-10 23:15:57 +0100 (Tue, 10 Mar 2009) $ / ($Author: rs $)
 */
public class SimVarParamExtracter {

	public static void extractSettings(SimulationResultData result, DataModel configuration, Map<DataIdentification, Data> newSettings, short simVar) {
		try {
			Data data = result.getData();
			Debug.getLogger().finer("Parametrierte Daten fuer Simulationsvariante " + result.getSimVar() + " = " + data);
			if(data == null) return;
			Data parameterArray = data.getItem("ParameterSatz");
			//Schleife ber die einzelnen Elemente des Arrays ParameterSatz
			for(Iterator iterator = parameterArray.iterator(); iterator.hasNext();) {
				Data parameter = (Data)iterator.next();
				SystemObject[] specifiedConfigAreas = parameter.getReferenceArray("Bereich").getSystemObjectArray();
				specifiedConfigAreas = removeNullReferences(specifiedConfigAreas);
				Data ack = parameter.getItem(K.QUITTIEREN);
				for(Iterator dataSpecificationIterator = parameter.getItem("Eingangsdaten").iterator(); dataSpecificationIterator.hasNext();) {
					Data dataSpec = (Data)dataSpecificationIterator.next();
					SystemObject[] specifiedObjects = dataSpec.getReferenceArray("Objekt").getSystemObjectArray();
					specifiedObjects = removeNullReferences(specifiedObjects);
					SystemObject[] specifiedAtgs = dataSpec.getReferenceArray("AttributGruppe").getSystemObjectArray();
					specifiedAtgs = removeNullReferences(specifiedAtgs);
					// Sortierung ist notwendig fr das spter binre Suchen in diesem Array s.u.
					Arrays.sort(specifiedAtgs);
					SystemObject[] specifiedAspects = dataSpec.getReferenceArray("Aspekt").getSystemObjectArray();
					// Sortierung ist notwendig fr das spter binre Suchen in diesem Array s.u.
					specifiedAspects = removeNullReferences(specifiedAspects);
					Arrays.sort(specifiedAspects);

					List objectList;
					if(specifiedObjects.length == 0) {
						// keine Objekte sind alle Objekte
						// d.h. alle dynamischen Objekte und alle Konfigurationsobjekte
						
						List dynamicObjects = configuration.getType(Pid.Type.DYNAMIC_OBJECT).getObjects();
						List configurationObjects = configuration.getType(Pid.Type.CONFIGURATION_OBJECT).getObjects();
						objectList = new ArrayList(dynamicObjects.size() + configurationObjects.size());
						objectList.addAll(dynamicObjects);
						objectList.addAll(configurationObjects);
					}
					else {
						objectList = new ArrayList(specifiedObjects.length);
						for(int i = 0; i < specifiedObjects.length; i++) {
							SystemObject object = specifiedObjects[i];
							if(object instanceof SystemObjectCollection) {
								SystemObjectCollection collection = (SystemObjectCollection)object;
								// Typen und Mengen werden durch die jeweils enthaltenen Elemente ersetzt
								objectList.addAll(collection.getElements());
							}
							else {
								objectList.add(object);
							}
						}
					}

					// Einschrnkung auf ausgewhlte Konfigurationsbereiche
					if(specifiedConfigAreas.length > 0) {
						List specifiedConfigAreaList = Arrays.asList(specifiedConfigAreas);
						List newObjectList = new LinkedList();
						for(Iterator objectIterator = objectList.iterator(); objectIterator.hasNext();) {
							SystemObject systemObject = (SystemObject)objectIterator.next();
							ConfigurationArea configurationArea = systemObject.getConfigurationArea();
							if(specifiedConfigAreaList.contains(configurationArea)) {
								newObjectList.add(systemObject);
							} // alle anderen werden aus der Objektliste gelscht
						}
						objectList.clear();
						objectList.addAll(newObjectList);
					}

					// Folgende Map Enthlt als Schlssel die zu betrachtenden Typen und als Wert jeweils ein Set mit den zu
					// betrachtenden Objekten des Typs:
					Map type2objectSetMap = new TreeMap();

					// Erzeugen der Map, d.h. sortieren nach Typen und sicherstellen, dass jedes Objekt nur einmal enthalten ist.
					for(Iterator objectIterator = objectList.iterator(); objectIterator.hasNext();) {
						SystemObject object = (SystemObject)objectIterator.next();
						SystemObjectType type = object.getType();
						Collection objectCollection = (Collection)type2objectSetMap.get(type);
						if(objectCollection == null) {
							objectCollection = new LinkedList();
							type2objectSetMap.put(type, objectCollection);
						}
						objectCollection.add(object);
					}

					objectList.clear();
					objectList = null;

					
					for(Iterator mapIterator = type2objectSetMap.entrySet().iterator(); mapIterator.hasNext();) {
						Map.Entry entry = (Map.Entry)mapIterator.next();
						SystemObjectType type = (SystemObjectType)entry.getKey();
						Collection objectCollection = (Collection)entry.getValue();
						// Im folgenden werden alle Attributgruppen des Typs betrachtet und diese eventuell weiter
						// eingeschrnkt, dadurch werden unsinnige Spezifikation ignoriert.
						List typeAtgs = type.getAttributeGroups();
						for(Iterator atgIterator = typeAtgs.iterator(); atgIterator.hasNext();) {
							AttributeGroup typeAtg = (AttributeGroup)atgIterator.next();
							// Konfigurierende Attributgruppen werden ignoriert
							if(typeAtg.isConfigurating()) continue;
							

							// Wenn keine Attributgruppen spezifiziert wurden, dann werden alle des Typs weiter betrachtet,
							// ansonsten werden nur die Attributgruppen des Typs weiter betrachtet, die auch explizit
							// spezifiziert wurden.
							if(specifiedAtgs.length > 0 && Arrays.binarySearch(specifiedAtgs, typeAtg) < 0) continue;

							Collection atgAspects = typeAtg.getAspects();

							for(Iterator aspectIterator = atgAspects.iterator(); aspectIterator.hasNext();) {
								Aspect atgAspect = (Aspect)aspectIterator.next();
								// Wenn kein Aspekt spezifiziert wurde, dann werden alle Aspekte der Attributgruppe weiter betrachtet,
								// ansonsten werden nur die Aspekte weiter betrachtet, die auch explizit
								// spezifiziert wurden.
								if(specifiedAspects.length > 0 && Arrays.binarySearch(specifiedAspects, atgAspect) < 0) continue;

								// Fr alle spezifizierten Objekte des betrachteten Typs die spezifizierte Einstellung merken
								for(Iterator objectIterator = objectCollection.iterator(); objectIterator.hasNext();) {
									SystemObject object = (SystemObject)objectIterator.next();
									DataIdentification key = new DataIdentification(object, new DataDescription(typeAtg, atgAspect, simVar));
									newSettings.put(key, ack);
								}
							}
						}
					}
				}
			}
		}
		catch(ConfigurationException e) {
			throw new RuntimeException(e);
		}
	}


	private static SystemObject[] removeNullReferences(SystemObject[] objects) {
		int numberOfNonNullObjects = 0;
		for(int i = 0; i < objects.length; i++) {
			if(objects[i] != null) numberOfNonNullObjects++;
		}
		if(numberOfNonNullObjects != objects.length) {
			SystemObject[] trimmedObjects = new SystemObject[numberOfNonNullObjects];
			int trimmedIndex = 0;
			for(int i = 0; i < objects.length; i++) {
				if(objects[i] != null) trimmedObjects[trimmedIndex++] = objects[i];
			}
			return trimmedObjects;
		}
		else {
			return objects;
		}
	}
}
