/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.ars.ars.mgmt.tasks;

import de.bsvrz.ars.ars.mgmt.ArchiveManager;
import de.bsvrz.ars.ars.mgmt.tasks.tasklistener.RingOfDeathListener;
import de.bsvrz.ars.ars.mgmt.tasks.tasklistener.TaskStepListener;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sys.funclib.losb.ringbuffer.RingBuffer;

/**
 * SingleTask. Abstrakte Basisklasse fuer diejenigen Tasks, die nur einmal vorkommen und deswegen nicht von einem TaskMgr verwaltet sondern direkt durch die
 * Eingabe-Warteschlange gesteuert werden.
 *
 * @author beck et al. projects GmbH
 * @author Alexander Schmidt
 * @version $Revision: 7744 $ / $Date: 2010-06-01 19:35:01 +0200 (Tue, 01 Jun 2010) $ / ($Author: rs $)
 */
public abstract class SingleTask extends Task {

	private RingBuffer<ResultData> inputDataQueue;

	/**
	 * Erzeugt eine neue SingleTask.
	 *
	 * @param archiveMgr Der Archiv-Manager.
	 * @param rodl       Der Listener wird benachrichtigt beim Beenden der Task.
	 * @param inputData  Eingangswarteschlange
	 */
	public SingleTask(ArchiveManager archiveMgr, RingOfDeathListener rodl, RingBuffer<ResultData> inputData) {
		super(archiveMgr, rodl);
		inputDataQueue = inputData;
	}

	/** @see java.lang.Runnable#run() */
	public void run() {
		try {
			while(!isTaskTerminated()) {
				try {
					work(inputDataQueue.pop());	// synchronized, blockiert falls leer

					synchronized(taskStepListeners) {
						for(TaskStepListener tsl : taskStepListeners) {
							tsl.taskStepDone(this);
						}
					}
				}
				catch(InterruptedException e) {
					break;
				}
				catch(Exception e) {
					logger.error("Fehler", e);
				}
			}
		}
		finally {
			if(ringOfDeathListener != null) ringOfDeathListener.ringOfDeath(this);
			isTerminated = true;
			inputDataQueue = null;
		}
	}

	/**
	 * Diese Methode ist zu ueberschreiben, um die konkrete Aufgabe der Task zu implementieren. In der Methode soll keine weitere Warteschlangen-Synchronisation
	 * stattfinden.
	 *
	 * @param resultData Zu verarbeitender Datensatz aus der Eingangswarteschlange
	 */
	protected abstract void work(ResultData resultData);
    
}
